# filter error/warning messages from GNU assembler
#

select((select(STDERR), $| = 1)[0]);    # unbuffer output
select((select(STDOUT), $| = 1)[0]);

sub usage {
    if ($ARGV[0] eq '') {
        print STDERR "usage: $0 avr-as file.s file.lst file.o [options]\n";
        exit(1);
    }
}

$debug = 0;
$unix = 0;
$no_spawn = 0;

$avr_as = $ARGV[0]; &usage if ($avr_as eq ''); shift;
$asm_file = $ARGV[0]; &usage if ($asm_file eq ''); shift;
$lst_file = $ARGV[0]; &usage if ($lst_file eq ''); shift;
$obj_file = $ARGV[0]; &usage if ($obj_file eq ''); shift;

while ($ARGV[0] ne '') {
    if ($ARGV[0] eq '-u') { $unix = 1; shift; next; }
    if ($ARGV[0] eq '-d') { $debug = 1; shift; next; }
    if ($ARGV[0] eq '-ns') { $no_spawn = 1; shift; next; }
    if ($ARGV[0] =~ /^-D/) { shift; next; }
    $parms .= $ARGV[0] . ' '; shift
}

###print STDERR "$parms\n"; exit 1;

$asm_cmd = "$avr_as -W $parms -alms=$lst_file -o $obj_file $asm_file";
print "$asm_cmd\n";

open(A,"<$asm_file") || die "$asm_file: $!" ;
$aline = 0 ;
select((select(STDERR), $| = 1)[0]);    # unbuffer output

$dir = $0;
$dir =~ s/([\\\/])[^\\\/]+$/$1/;
if ($unix) {
	$cmd = $asm_cmd;
	$no_spawn = 1;
} else {
	$cmd = $dir.$asm_cmd;
	$spawn = $dir.'spawn';
}

if ($no_spawn) {   # testing "hook"
    $spawn = '';
}

$cmd = "$spawn \"$cmd\"" if ($spawn ne '');
print STDERR "$cmd\n" if ($debug);
open(F,"$cmd|") || die "avrasm" ;

$n_err = $errn = 0 ;
$curdir = `cd` ;
$curdir =~ s/[\r\n]//g ;
$curdir .= "\\" ;
$n_err = 0 ;
while (<F>)  {
    ###if (s/^([^\(]+)\(([^\)]+)\):/&file_line($1,$2):/e)  {
    if (s/^([^\:]+):(\d+):/&file_line($1,$2)/e)  {
        $n_err++ ;
        print ;
    } elsif (/^\s*$/)  {
        next ;
    } else {
        print ;
    }
}
if (! -f $obj_file) {
    if ($n_err == 0) {
	$n_err = 1 ;
	print "Error: no output file ($obj_file) produced!\n" ;
    }
    exit(1) ;
}
exit(0) ;

sub file_line  {
    local($file,$line,$rec) = @_ ;   # file name/line number
    while ($aline < $line)  {   # need to advance
        $rec = scalar(<A>) ;    # get next record
        if ($rec =~ /^#line\s+([0-9]+)\s+\"([^\"]+)\"/)  {   # #line directive
            $fn = $2 ;
            $fline = $1-1 ;
        }
        elsif ($rec =~ /^#line/)  {
            print "Bogus: $rec" ;
        }
        else  {
            $fline++ ;
        }
        $aline++ ;
    }
    $fn =~ s/\//\\/g;
    if ($fn =~ /^[.]\\(.*)/) {
        $fn = $curdir.$1;
    } elsif ($fn =~ /^([.][.]\\.*)/) {
        $fn = $curdir.$1;
    }
    "$fn:$fline($asm_file:$aline)" ;
}

