#! /usr/local/perl
#
#  Convert Ethernet (MAC) address (dot notation) to "ether.inc"
#

sub usage {
	print STDERR "usage: $ego [-v] ddd.ddd.ddd.ddd.ddd.ddd\n";
	print STDERR "       $ego [-v] hh:hh:hh:hh:hh:hh\n";
	print STDERR "       $ego [-v] \@file\n";
	print STDERR "       $ego [-v] - < file\n";
	exit 1;
}

$ego = $0;
$ego =~ s/.*[\/\\]//;

$verbose = 0;
while ($ARGV[0] =~ /^-/ && $ARGV[0] ne '-') {
	$opt = shift;
	if ($opt eq '-v') {
		$verbose = 1;
	} else {
		&usage;
	}
}

$mac = shift;
&usage if ($mac eq '');

exit(1) if (($mac = &getmac($mac)) eq '');

(@a) = split(/\./,$mac) ;
$mac = sprintf("%d.%d.%d.%d.%d.%d", $a[0], $a[1], $a[2], $a[3], $a[4], $a[5]);

printf("\t.dw\t0x%02x%02x\t; Ethernet address $mac\n", $a[1], $a[0]);
printf("\t.dw\t0x%02x%02x\n", $a[3], $a[2]);
printf("\t.dw\t0x%02x%02x\n", $a[5], $a[4]);

print STDERR sprintf("$mac (%x:%x:%x:%x:%x:%x)\n",$a[0],$a[1],$a[2],$a[3],$a[4],$a[5]) if ($verbose);

#
#  Resolve MAC address
#  usage: exit(1) if (($mac = &getmac($mac)) eq '');
#         where $s is MAC address, '-', or @file
#
sub getmac {
    local($f) = @_;
    local($s);

    $mac = '';
    $mac = $ENV{'PW_MAC'} if ($ENV{'PW_MAC'} ne '');
    if ($mac ne '') {
        ;
    } elsif ($f =~ /^[@]/ || $f eq '-') {
        if ($f ne '-') {
            $f =~ s/^[@]//;
            if (!open(MAC, "<$f")) {
                print STDERR "Can't open MAC address file \"$f\"!\n";
                return '';
            }
        }
        while(1) {
            if ($f ne '-') {
                last if (!($s = <MAC>));
            } else {
                last if (!($s = <>));
            }
            ##print STDERR "$s";
            $s =~ s/[\s\r\n]+$//;
            $s =~ s/\s*[#].*$//;
            next if ($s eq '');
            if ($s =~ /^\s*([0-9]+[.][0-9]+[.][0-9]+[.][0-9]+[.][0-9]+[.][0-9]+)$/i) {
                $mac = $s;
            }
            if ($s =~ /^\s*([0-9a-f]+[:][0-9a-f]+[:][0-9a-f]+[:][0-9a-f]+[:][0-9a-f]+[:][0-9a-f]+)$/i) {
                $mac = $s;
            }
        }
    } else {
        $mac = $f;
    }
    if ($mac eq '') {
	print STDERR "Can't find Ethernet address of the form ddd.ddd.ddd.ddd.ddd.ddd or hh:hh:hh:hh:hh:hh!\n";
    } else {
        ##print STDERR "MAC/NM=$mac\n";
        if ($mac =~ /^\d+[.]\d+[.]\d+[.]\d+[.]\d+[.]\d+$/) {
            return $mac;
        } elsif ($mac =~ /^\s*([0-9a-f]+)[:]([0-9a-f]+)[:]([0-9a-f]+)[:]([0-9a-f]+)[:]([0-9a-f]+)[:]([0-9a-f]+)$/i) {
            $mac = sprintf("%d.%d.%d.%d.%d.%d",hex($1),hex($2),hex($3),hex($4),hex($5),hex($6));
            return $mac;
        } else {
	    print STDERR "Ethernet address ($mac) not of the form ddd.ddd.ddd.ddd.ddd.ddd or hh:hh:hh:hh:hh:hh!\n";
        }
    }
    return ('', '');   # error return
}
