;-------------------------------------------------------------------------
; Copyright (c) 1999 Steven Freyder, David Helland, Bruce D. Lightner and
; Lightner Engineering.  Use subject to licensing restrictions described
; the file LICENSE.txt.  If you do not agree with the licensing terms then
; you must return and/or destroy this software immediately!
;-------------------------------------------------------------------------

#if !defined(DS1621_BASE_ADDR)
#define DS1621_BASE_ADDR 0x9E    /* ext. DS1621 digital thermometer */
#endif

#if !defined(SER_EEPROM_BASE_ADDR)
#define SER_EEPROM_BASE_ADDR 0xa0
#endif

.global ser_eeprom_base_addr
.equ ser_eeprom_base_addr,SER_EEPROM_BASE_ADDR

.dseg
i2c_last_addr:   .word 0xffff           ; last address for burst mode
.cseg

;**** Global I2C Constants ****

#define USE_I2C_LAST_ADDR
;;;#define LOG_SEEPROM_ACCESS
;;;#define LOG_SEEPROM_HITS

.equ    SDAP0    = 0                    ; SDA0 Pin number
.equ    SDAPORT0 = PORTA                ; SDA0 port register
.equ    SDAPIN0  = PINA                 ; SDA0 pin register
.equ    SDADDR0  = DDRA                 ; SDA0 data direction register

#ifdef I2C_PORT1_PRESENT
#ifdef PICOWEB_PROTO
.equ    SDAP1    = 6                    ; SDA1 Pin number
#else
.equ    SDAP1    = 2                    ; SDA1 Pin number
#endif
.equ    SDAPORT1 = PORTD                ; SDA1 port register
.equ    SDAPIN1  = PIND                 ; SDA1 pin register
.equ    SDADDR1  = DDRD                 ; SDA1 data direction register
#endif

.equ    SCLP     = 7                    ; SCL Pin number (port D)
.equ    SCLPORT  = PORTD                ; SCL port register
.equ    SCLPIN   = PIND                 ; SCL pin register
.equ    SCLDDR   = DDRD                 ; SCL data direction register

.equ    b_dir   = 0                     ; transfer direction bit in i2cadr

.equ    i2crd   = 1
.equ    i2cwr   = 0

.macro SDA_HIZ
	cbi     SDADDR0,SDAP0   ; make input
	sbi     SDAPORT0,SDAP0  ; enable pullup
	;;cbi     SDAPORT0,SDAP0  ; disable pullup
#ifdef I2C_PORT1_PRESENT
	cbi     SDADDR1,SDAP1   ; make input
	sbi     SDAPORT1,SDAP1  ; enable pullup
	;;cbi     SDAPORT1,SDAP1  ; disable pullup
#endif
.endm

.macro SDA_ACTIVE_LOW
	cbi     SDAPORT0,SDAP0
#ifdef I2C_PORT1_PRESENT
	cbi     SDAPORT1,SDAP1
#endif
.endm

.macro SDA_LOW
	cbi     SDAPORT0,SDAP0  ; disable pullup
	sbi     SDADDR0,SDAP0   ; make output
#ifdef I2C_PORT1_PRESENT
	cbi     SDAPORT1,SDAP1  ; disable pullup
	sbi     SDADDR1,SDAP1   ; make output
#endif
.endm

.macro SDA_TO_CF
	sec                     ; set carry flag
	sbis    SDAPIN0,SDAP0   ; if SDA is low
	clc                     ;       clear carry flag
#ifdef I2C_PORT1_PRESENT
	sbis    SDAPIN1,SDAP1   ; if other SDA is low
	clc                     ;       clear carry flag
#endif
.endm

.macro SCL_HIZ
	cbi     SCLDDR,SCLP
.endm

.macro SCL_ACTIVE_LOW
	cbi     SCLPORT,SCLP
.endm

.macro SCL_LOW
	sbi     SCLDDR,SCLP
.endm

;;;.macro SKIP_IF_SCL_HI
;;;     sbis    SCLPIN,SCLP
;;;.endm

#if 0
.macro BR_IF_SCL_LOW p0
	lds     w,(SCLPIN+$20)  ; load from I/O register
	bst     w,SCLP          ; load bit into T-flag
	brtc    \p0              ; branch if T-flag is clear
.endm
#else
.macro BR_IF_SCL_LOW p0
	sbis    SCLPIN,SCLP     ; skip if SCL high
	rjmp    \p0              ; branch if SCL low
.endm
#endif

;*****************************
;* Global Register Variables *
;*****************************

;**** Global I2C Register Variables ****

.equ    i2cdata  , r17                  ; I2C data transfer register
.equ    i2cadr   , r18                  ; I2C address and direction register
.equ    i2ccnt   , r19                  ; I2C transfer byte counter
#define SEE_SAVE

;**** A P P L I C A T I O N   N O T E   A V R 3 0 0 ************************
;*
;* Title                : I2C (Single) Master Implementation
;* Version              : 1.20 (BETA)
;* Last updated         : 97.08.27
;* Target               : All AVR device
;*
;* Support email        : avr@atmel.com
;*
;* DESCRIPTION
;*      Basic routines for communicating with I2C slave devices. This
;*      "single" master implementation is limited to one bus master on the
;*      I2C bus. Most applications do not need the multimaster ability
;*      the I2C bus provides. A single master implementation uses, by far,
;*      less resources and is less XTAL frequency dependent.
;*
;*      Some features :
;*      * All interrupts are free, and can be used for other activities.
;*      * Supports normal and fast mode.
;*      * Supports both 7-bit and 10-bit addressing.
;*      * Supports the entire AVR microcontroller family.
;*
;*      Main I2C functions :
;*      'i2c_start' -           Issues a start condition and sends address
;*                              and transfer direction.
;*      'i2c_rep_start' -       Issues a repeated start condition and sends
;*                              address and transfer direction.
;*      'i2c_stop' -            Terminates the data transfer by issue a
;*                              stop condition.
;*
;* USAGE
;*      Transfer formats is described in the AVR300 documentation.
;*      (An example is shown in the 'main' code).       
;*
;* NOTES
;*      The I2C routines can be called either from non-interrupt or
;*      interrupt routines, not both.
;*
;* STATISTICS
;*      Code Size       : 81 words (maximum)
;*      Register Usage  : 4 High, 0 Low
;*      Interrupt Usage : None
;*      Other Usage     : Uses two I/O pins on port D
;*      XTAL Range      : N/A
;*
;***************************************************************************


;***************************************************************************
;*
;* FUNCTION
;*      i2c_hp_delay
;*      i2c_qp_delay
;*
;* DESCRIPTION
;*      hp - half i2c clock period delay (normal: 5.0us / fast: 1.3us)
;*      qp - quarter i2c clock period delay (normal: 2.5us / fast: 0.6us)
;*
;*      SEE DOCUMENTATION !!!
;*
;* USAGE
;*      no parameters
;*
;* RETURN
;*      none
;*
;***************************************************************************

#ifdef I2C_DELAY_OLD_WAY
i2c_hp_delay:
	push    r16
	ldi     r16,2
i2c_hp_delay_loop:
	dec     r16
	brne    i2c_hp_delay_loop
	pop     r16
	ret

i2c_qp_delay:
	push    r16
	ldi     r16,1
i2c_qp_delay_loop:
	dec     r16
	brne    i2c_qp_delay_loop
	pop     r16
	ret
#else
;
;  Required delays for FAST mode are:
;     Half cycle delay  8Mhz/400Khz/2=10 cycles (1.25uS)
;     Quarter cycle delay=5 cycles (0.625uS)
;
.global i2c_hp_delay
i2c_hp_delay:
	rjmp    i2c_hp_delay_dummy
i2c_hp_delay_dummy:
	ret

i2c_qp_delay:
	ret
#endif

;***************************************************************************
;*
;* FUNCTION
;*      i2c_rep_start
;*
;* DESCRIPTION
;*      Assert repeated start condition and sends slave address.
;*
;* USAGE
;*      i2cadr - Contains the slave address and transfer direction.
;*
;* RETURN
;*      Carry flag - Cleared if a slave responds to the address.
;*
;* NOTE
;*      IMPORTANT! : This funtion must be directly followed by i2c_start.
;*
;***************************************************************************

.global i2c_rep_start
i2c_rep_start:
	SCL_LOW                         ; force SCL low
	SDA_HIZ                         ; release SDA
	rcall   i2c_hp_delay            ; half period delay
	SCL_HIZ                         ; release SCL
	rcall   i2c_qp_delay            ; quarter period delay

;***************************************************************************
;*
;* FUNCTION
;*      i2c_start
;*
;* DESCRIPTION
;*      Generates start condition and sends slave address.
;*
;* USAGE
;*      i2cadr - Contains the slave address and transfer direction.
;*
;* RETURN
;*      Carry flag - Cleared if a slave responds to the address.
;*
;* NOTE
;*      IMPORTANT! : This funtion must be directly followed by i2c_write.
;*
;***************************************************************************

.global i2c_start
i2c_start:
	mov     i2cdata,i2cadr          ; copy address to transmit register
	SDA_LOW                         ; force SDA low
	rcall   i2c_qp_delay            ; quarter period delay

;***************************************************************************
;*
;* FUNCTION
;*      i2c_write
;*
;* DESCRIPTION
;*      Writes data (one byte) to the I2C bus. Also used for sending
;*      the address.
;*
;* USAGE
;*      i2cdata - Contains data to be transmitted.
;*
;* RETURN
;*      Carry flag - Set if the slave respond transfer.
;*
;* NOTE
;*      IMPORTANT! : This funtion must be directly followed by i2c_get_ack.
;*
;***************************************************************************

.global i2c_write
i2c_write:
	sec                             ; set carry flag
	rol     i2cdata                 ; shift in carry and out bit one
	rjmp    i2c_write_first
i2c_write_bit:
	lsl     i2cdata                 ; if transmit register empty
i2c_write_first:
	breq    i2c_get_ack             ;       goto get acknowledge
	SCL_LOW                         ; force SCL low

	brcc    i2c_write_low           ; if bit high
	nop                             ;       (equalize number of cycles)
	SDA_HIZ                         ;       release SDA
	rjmp    i2c_write_high
i2c_write_low:                          ; else
	SDA_LOW                         ;       force SDA low
	rjmp    i2c_write_high          ;       (equalize number of cycles)
i2c_write_high:
	rcall   i2c_hp_delay            ; half period delay
	SCL_HIZ         ; release SCL
	rcall   i2c_hp_delay            ; half period delay

	rjmp    i2c_write_bit

;***************************************************************************
;*
;* FUNCTION
;*      i2c_get_ack
;*
;* DESCRIPTION
;*      Get slave acknowledge response.
;*
;* USAGE
;*      (used only by i2c_write in this version)
;*
;* RETURN
;*      Carry flag - Cleared if a slave responds to a request.
;*
;***************************************************************************

.global i2c_get_ack
i2c_get_ack:
	SCL_LOW                         ; force SCL low
	SDA_HIZ                         ; release SDA
	rcall   i2c_hp_delay            ; half period delay
	SCL_HIZ                         ; release SCL

i2c_get_ack_wait:
	BR_IF_SCL_LOW i2c_get_ack_wait  ; wait SCL high  (wait states?)

	SDA_TO_CF                       ; SDA state to carry flag

	rjmp    i2c_hp_delay            ; half period delay
	;ret implied

;***************************************************************************
;*
;* FUNCTION
;*      i2c_read
;*
;* DESCRIPTION
;*      Reads data (one byte) from the I2C bus.
;*
;* USAGE
;*      Carry flag -    If set no acknowledge is given to the slave
;*                      indicating last read operation before a STOP.
;*                      If cleared acknowledge is given to the slave
;*                      indicating more data.
;*
;* RETURN
;*      i2cdata - Contains received data.
;*
;* NOTE
;*      IMPORTANT! : This funtion must be directly followed by i2c_put_ack.
;*
;***************************************************************************

.global i2c_read
i2c_read:
	ror     i2cadr                  ; store acknowledge
	                                ; (used by i2c_put_ack)
	ldi     i2cdata,0x01            ; data = 0x01
i2c_read_bit:                           ; do
	SCL_LOW                         ;       force SCL low
	rcall   i2c_hp_delay            ;       half period delay

	SCL_HIZ                         ;       release SCL
	rcall   i2c_hp_delay            ;       half period delay

	SDA_TO_CF                       ; SDA state to carry flag

	rol     i2cdata                 ;       store data bit
	brcc    i2c_read_bit            ; while receive register not full

;***************************************************************************
;*
;* FUNCTION
;*      i2c_put_ack
;*
;* DESCRIPTION
;*      Put acknowledge.
;*
;* USAGE
;*      (used only by i2c_read in this version)
;*
;* RETURN
;*      none
;*
;***************************************************************************

.global i2c_put_ack
i2c_put_ack:
	SCL_LOW                         ; force SCL low

	clc                             ; clear carry (i2adr bit 0 -> 00)
	rol     i2cadr                  ; move status bit back into carry
	brcc    i2c_put_ack_low         ; if bit low goto assert low
	SDA_HIZ                         ;       release SDA
	rjmp    i2c_put_ack_high
i2c_put_ack_low:                        ; else
	SDA_LOW                         ;       force SDA low
i2c_put_ack_high:

	rcall   i2c_hp_delay            ; half period delay
	SCL_HIZ                         ; release SCL
i2c_put_ack_wait:
	BR_IF_SCL_LOW i2c_put_ack_wait  ; wait SCL high  (wait states?)
	rcall   i2c_hp_delay            ; half period delay

	SCL_LOW                         ; force SCL low
	SDA_HIZ                         ;       release SDA

	ret

;***************************************************************************
;*
;* FUNCTION
;*      i2c_stop
;*
;* DESCRIPTION
;*      Assert stop condition.
;*
;* USAGE
;*      No parameters.
;*
;* RETURN
;*      None.
;*
;***************************************************************************

.global i2c_stop
i2c_stop:
	SCL_LOW                         ; force SCL low
	SDA_LOW                         ; force SDA low
	rcall   i2c_hp_delay            ; half period delay
	SCL_HIZ                         ; release SCL
	rcall   i2c_qp_delay            ; quarter period delay
	SDA_HIZ                         ; release SDA
	rjmp    i2c_hp_delay            ; half period delay
	;ret implied

;***************************************************************************
;*
;* FUNCTION
;*      i2c_init
;*
;* DESCRIPTION
;*      Initialization of the I2C bus interface.
;*
;* USAGE
;*      Call this function once to initialize the I2C bus. No parameters
;*      are required.
;*
;* RETURN
;*      NONE
;*
;* NOTE
;*      PORTD and DDRD pins not used by the I2C bus interface will be
;*      unaffected.
;*
;* COMMENT
;*      This function can be combined with other PORTD initializations.
;*
;***************************************************************************


.global i2c_init
i2c_init:
    .section reset_io
        SCL_HIZ                         ; make sure SDLP is an input
        SCL_ACTIVE_LOW                  ; drive low when not set hi-z
        SDA_HIZ                         ; make sure SDAP is an input
        SDA_ACTIVE_LOW                  ; drive low when not set hi-z

        clrw y                          ; finish any active transfer
        rcall rdseeb                    ; in case we reset mid-transfer!

    .text

;***************************************************************************
;*
;* FUNCTION
;*      i2c_set_addr
;*
;* DESCRIPTION
;*      Write device address to I2C device
;*
;* USAGE
;*      i2cadr has device address + (i2cwr or i2crd)
;*      y has address in serial EEPROM (one byte address of yh=0xff)
;*
;* RETURN
;*      y = y + 1
;*      (i2c_last_addr) = y + 1
;*
;***************************************************************************

.global i2c_set_addr
i2c_set_addr:
	SDA_HIZ                         ; make sure SDAP is an input
	;;;;SDA_ACTIVE_LOW              ; drive low when not set hi-z

	rcall   i2c_start               ; Send start condition and address

	cpi     yh,0xff                 ; high byte of y all ones?
	breq    i2c_set_addr1           ;   yes...single address byte

	mov     i2cdata,yh              ; Write word address (hi byte)
	andi    i2cdata,0x7f            ; mask off p-code indirect bit
	rcall   i2c_write               ; Execute transfer

	mov     i2cdata,yl              ; Write word address (low byte)
        adiw    yl,1                    ; advance address

	rjmp    i2c_write               ; Execute transfer

i2c_set_addr1:
	mov     i2cdata,yl              ; Write word address (low byte)
	rjmp    i2c_write               ; Execute transfer

;***************************************************************************
;*
;* FUNCTION
;*      i2cwrite_bytes
;*
;* DESCRIPTION
;*      Write multiple data bytes to an address in serial EEPROM
;*
;* USAGE
;*      i0 has byte count (0 means poll)
;*      i2cadr has device address
;*      y has address in serial EEPROM (one byte address if yh=0xff)
;*      x has SRAM address
;*
;* RETURN
;*      y = y + byte count
;*      x = x + byte count
;*
;* COMMENT
;*
;***************************************************************************

.global i2cwrite_bytes
i2cwrite_bytes:

#if defined(SEE_SAVE)
	push i2cdata
	push i2ccnt
#endif
	mov     i2ccnt,i0               ; save count
	rcall   i2c_set_addr            ; send out address to SEEPROM
	brcs    i2cwrite3               ; immediate return if no response
	tst     i2ccnt                  ; poll? (i.e., count == 0?)
	breq    i2cwrite2               ;   yes...take early exit (save y)
	decw    y                       ; adjust y

i2cwrite_bytes2:
	ld      i0,x+                   ; get next data byte
	mov     i2cdata,i0              ; Set write data per i0
	rcall   i2c_write               ; Execute transfer
        adiw    yl,1                    ; advance address
	dec     i2ccnt                  ; decrement count
	brne    i2cwrite_bytes2         ; loop back if not zero
	rjmp    i2cwrite2

;***************************************************************************
;*
;* FUNCTION
;*      i2cwrite
;*
;* DESCRIPTION
;*      Write data byte to an address in serial EEPROM
;*
;* USAGE
;*      i2cadr has device address
;*      i0 has data byte to write
;*      y has address in serial EEPROM (one byte address if yh=0xff)
;*
;* RETURN
;*      y = y + 1
;*
;* COMMENT
;*      This function is inefficient because the address is re-sent for
;*      each byte written!!!
;*
;***************************************************************************

.global i2cwrite
i2cwrite:

#if defined(SEE_SAVE)
	push    i2cdata
	push    i2ccnt
#endif
	rcall   i2c_set_addr

	mov     i2cdata,i0              ; Set write data per i0
	rcall   i2c_write               ; Execute transfer

i2cwrite2:
#ifdef USE_I2C_LAST_ADDR
	rcall i2c_save_last             ; save y in i2c_last_addr
#endif

i2cwrite3:
	rol     i2cdata                 ; save carry flag
	rcall   i2c_stop                ; Send stop condition - releases bus
	ror     i2cdata                 ; restore carry flag
i2cwrite_done:
#if defined(SEE_SAVE)
	pop     i2ccnt
	pop     i2cdata
#endif
	ret

;***************************************************************************
;*
;* FUNCTION
;*      i2cread
;*
;* DESCRIPTION
;*      Read a data byte from an address in serial EEPROM
;*
;* USAGE
;*      i0 has byte count
;*      i2cadr has device address
;*      y has address in serial EEPROM (one byte address if yh=0xff)
;*      z routine to call to receive read data byte in i0
;*
;* RETURN
;*      i0 = last byte read
;*      y = y + byte count
;*
;* COMMENT
;*
;***************************************************************************

.global i2cread
i2cread:
#if defined(SEE_SAVE)
	push    i2cdata
	push    i2ccnt
#endif
	mov     i2ccnt,i0               ; save passed byte count
	andi    i2cadr,0xfe

#ifdef USE_I2C_LAST_ADDR
	cpi     yh,0xff                 ; high byte of y all ones?
	breq    i2cread0                ;   yes...single address byte

	lds     i2cdata,i2c_last_addr   ; check if y == i2c_last_addr?
	cp      yl,i2cdata
	brne    i2cread0                ; LSB mismatch...slow way

	lds     i2cdata,i2c_last_addr+1
	sub     i2cdata,yh
	andi    i2cdata,0x7f            ; mask off p-code indirect bit
	brne    i2cread0                ; MSB mismatch...slow way

#ifdef LOG_SEEPROM_HITS
	rcall   log_seeprom_hits
#endif
        adiw    yl,1                    ; bump address (as i2c_set_addr would)

	SDA_HIZ                         ; make sure SDAP is an input ;;;;;;;

	ori     i2cadr,i2crd
	rcall   i2c_start               ; Send start condition and control byte
	rjmp    i2cread1

i2c_save_last:
	sts     i2c_last_addr,yl        ; save y in i2c_last_addr
	sts     i2c_last_addr+1,yh
	ret
i2cread0:
#endif

	rcall   i2c_set_addr            ; send address byte(s)
	ori     i2cadr,i2crd
	rcall   i2c_rep_start           ; Send repeated start condition and address

i2cread1:
	decw    y                       ; adjust y
i2cread2:
	cpi     i2ccnt,1
	clc                             ; Set ack (read followed by read)
	brne    i2cread3
	sec                             ; Set no ack (read followed by stop)
i2cread3:
	rcall   i2c_read                ; Execute transfer (read)
	mov     i0,i2cdata              ; save read data byte
	icall                           ; call data byte consumer

        adiw    yl,1                    ; increment SEEPROM address
	dec     i2ccnt                  ; decrement count
	brne    i2cread2                ; go again if non-zero

#ifdef USE_I2C_LAST_ADDR
	rcall i2c_save_last             ; save y in i2c_last_addr
#endif
	rcall   i2c_stop                ; Send stop condition - releases bus
	rjmp    i2cwrite_done

;***************************************************************************
;+
; rdseeb - read a single byte from serial eeprom.
;
; inputs:
;   y   = eeprom address
;
; outputs:
;   y   = eeprom address + 1
;   i0  = data
;-
;***************************************************************************

.global import_seeprom_read
import_seeprom_read:
.global rdseeb

rdseeb:
	push    i2cadr
	pushw   z
	movwi   z,CODEP(rdseeb_ret)
	clr     i0                      ; byte count = 1
	inc     i0
	ldi     i2cadr,ser_eeprom_base_addr
	rcall   i2cread
	popw    z
	pop     i2cadr
rdseeb_ret:
	ret

;***************************************************************************
;+
; **-psee2s-read bytes from serial eeprom.
;
; inputs:
;    r10 = SRAM address
;    r12 = seeprom address
;    r14l = byte count
;
; outputs:
;    bytes read from seeprom
;-
;***************************************************************************
;+
; see2s - read multiple bytes from serial eeprom.
;
; inputs:
;   y   = eeprom address
;   x   = buffer
;   r18l = byte count
;
; outputs:
;   y   = y + byte count
;   x   = x + byte count
;   r18l = 0
;-
;***************************************************************************

        .global psee2s
        .global _see2s
psee2s:  pcode_routine   3
_see2s:
        rcall r10reloc
	movw    y,r12
	mov     r18l,r14l

.global import_seeprom_read_bytes
import_seeprom_read_bytes:
        .global see2s
see2s:
#ifdef LOG_SEEPROM_ACCESS
	rcall   log_seeprom_access
#endif
	mov     i0,r18l         ; move byte count to i0
	pushw   z
	movwi   z,CODEP(seeprom_read_stuff_byte)
	push    i2cadr
	ldi     i2cadr,ser_eeprom_base_addr
	rcall   i2cread         ; read data
	pop     i2cadr
	popw    z
	ret

seeprom_read_stuff_byte:
	st      x+,i0
	ret

;***************************************************************************
;+
; **-ps2see-write bytes to serial eeprom.
;
; inputs:
;    r10 = seeprom address
;    r12 = SRAM address
;    r14l = byte count
;
; outputs:
;    bytes written to seeprom
;-
;***************************************************************************
;+
; s2see - write multiple bytes to serial eeprom.
;
; inputs:
;   y   = eeprom address
;   x   = SRAM address
;   r18l = byte count (64 max.)
;
; outputs:
;   y   = eeprom address + r18l
;   x   = SRAM address + r18l
;-
;***************************************************************************

        .global ps2see
ps2see:  pcode_routine 3
	rcall	r12reloc
	movw    y,r10
	movw    x,r12
	mov     r18l,r14l
        .global s2see
s2see:
	mov     i0,r18l
	push    i2cadr
	ldi     i2cadr,ser_eeprom_base_addr
	rcall   i2cwrite_bytes
	rcall	i2cwait_done
	pop     i2cadr
	ret

i2cwait_done:
	clr     i0                      ; count=0 (indicate poll)
	ldi     i2cadr,ser_eeprom_base_addr
	rcall   i2cwrite_bytes
	brcs    i2cwait_done            ; busy if carry set (loop back)
	ret

;***************************************************************************
;+
; wrseebi - write a single byte to serial eeprom.
;
; inputs:
;   y   = eeprom address
;   i0  = data
;
; outputs:
;   y   = eeprom address + 1
;-
;***************************************************************************

.global wrseebi
wrseebi:
        push i2cadr
        ldi i2cadr,ser_eeprom_base_addr
        rcall i2cwrite
	rcall	i2cwait_done
        pop i2cadr
        ret

#ifdef I2C_PORT1_PRESENT
;***************************************************************************
;+
; ds1621_read - read a single byte from serial eeprom.
;
; inputs:
;   yl   = DS1621 register
;
; outputs:
;   y   = DS1621 register + 1
;   i0  = data
;-
;***************************************************************************

        .global ds1621_read
ds1621_read:    export
	push    i2cadr
	ldi     i2cadr,DS1621_BASE_ADDR
	ldi     yh,0xff
	pushw   z
	movwi   z,CODEP(ds1621_read_ret)
	clr     i0              ; byte count = 1
	inc     i0
	rcall   i2cread
	popw    z
	pop     i2cadr
ds1621_read_ret:
	ret

;***************************************************************************
;+
; ds1621_write - write a single byte to serial eeprom.
;
; inputs:
;   yl   = DS1621 register
;   i0  = data
;
; outputs:
;   y   = DS1621 register + 1
;-
;***************************************************************************

        .global ds1621_write
ds1621_write:   export
	push    i2cadr
	ldi     i2cadr,DS1621_BASE_ADDR
	ldi     yh,0xff
	rcall   i2cwrite
	pop     i2cadr
	ret

        .global pread_temp
pread_temp: pcode_routine 1
	ldi     yl,0xAA
	rcall   ds1621_read
        rcall r10reloc              ; x-> place to store result
	st      x+,i0
	ret
#endif  /* I2C_PORT1_PRESENT */


;***************************************************************************
;
;  SEEPROM Multiple-byte Reads into Registers on Base URL Fetch:
;
;       Mem     Cnt     Regs
;       ---     ---     ----
;       0x06     2      r6-7
;       0x07     1      r7
;       0x0A     2      r10-11
;       0x0A     4      r10-13
;       0x0B     1      r11
;       0x0B     3      r11-13
;       0x0C     2      r12-13
;       0x0D     1      r14
;       0x14     2      r20-21
;       0x14     4      r20-23
;       0x15     1      r21
;       0x15     3      r21-23
;       0x16     2      r22-23
;       0x17     1      r23
;
;  Summary:
;
;    r6-7
;    r10-14
;    r20-23
;
;***************************************************************************

#ifdef LOG_SEEPROM_ACCESS
; log SEEPROM access
log_seeprom_access:
	push r17        ; switch to serial port
	lds r17,putc_b
	push r17
	clr r17
	sts putc_b,r17

	putchar '-'     ; lead-in character

	mov i0,yh       ; print y
	hexb
	mov i0,yl
	hexb
	space

	mov i0,xh       ; print x
	hexb
	mov i0,xl
	hexb
	space

	mov i0,r18l     ; print count
	hexb
	crlf

	pop r17         ; switch back
	sts putc_b,r17
	pop r17

	ret
#endif

#ifdef LOG_SEEPROM_HITS
; log SEEPROM address hits
log_seeprom_hits:
	push r17        ; switch to serial port
	lds r17,putc_b
	push r17
	clr r17
	sts putc_b,r17

	putchar 'H'

	pop r17         ; switch back
	sts putc_b,r17
	pop r17

	ret
#endif
