#if !defined(MULDIV_INCLUDED)
#define MULDIV_INCLUDED
;**** A P P L I C A T I O N   N O T E   A V R 2 0 0 ************************
;*
;* Title:               Multiply and Divide Routines
;* Version:             1.2
;* Last updated:        97.07.04
;* Target:              All AVR Devices
;*
;* Support E-mail:      avr@atmel.com
;* ;* DESCRIPTION
;* This Application Note lists subroutines for the following
;* Muliply/Divide applications:
;*
;* 8x8 bit unsigned
;* 8x8 bit signed
;* 16x16 bit unsigned
;* 16x16 bit signed
;* 8/8 bit unsigned
;* 8/8 bit signed
;* 16/16 bit unsigned
;* 16/16 bit signed
;*
;* All routines are Code Size optimized implementations
;*;***************************************************************************

;***************************************************************************
;*
;* "mpy16u" - 16x16 Bit Unsigned Multiplication
;*
;* This subroutine multiplies the two 16-bit register variables 
;* mp16uh:mp16ul and mc16uh:mc16ul.
;* The result is placed in m16u3:m16u2:m16u1:m16u0.
;*  
;* Number of words      :14 + return
;* Number of cycles     :153 + return
;* Low registers used   :None
;* High registers used  :7 (mp16ul,mp16uh,mc16ul/m16u0,mc16uh/m16u1,m16u2,
;*                          m16u3,mcnt16u)      
;*
;***************************************************************************

;***** Subroutine Register Variables

.equ    mc16ul  ,r16            ;multiplicand low byte
.equ    mc16uh  ,r17            ;multiplicand high byte
.equ    mp16ul  ,r18            ;multiplier low byte
.equ    mp16uh  ,r19            ;multiplier high byte
.equ    m16u0   ,r18            ;result byte 0 (LSB)
.equ    m16u1   ,r19            ;result byte 1
.equ    m16u2   ,r20            ;result byte 2
.equ    m16u3   ,r21            ;result byte 3 (MSB)
.equ    mcnt16u ,r22            ;loop counter

;***** Code

.global mpy16u
mpy16u: clr     m16u3           ;clear 2 highest bytes of result
	clr     m16u2
	ldi     mcnt16u,16      ;init loop counter
	lsr     mp16uh
	ror     mp16ul

m16u_1: brcc    noad8           ;if bit 0 of multiplier set
	add     m16u2,mc16ul    ;add multiplicand Low to byte 2 of res
	adc     m16u3,mc16uh    ;add multiplicand high to byte 3 of res
noad8:  ror     m16u3           ;shift right result byte 3
	ror     m16u2           ;rotate right result byte 2
	ror     m16u1           ;rotate result byte 1 and multiplier High
	ror     m16u0           ;rotate result byte 0 and multiplier Low
	dec     mcnt16u         ;decrement loop counter
	brne    m16u_1          ;if not done, loop more
	ret
;***************************************************************************
;*
;* "div16u" - 16/16 Bit Unsigned Division
;*
;* This subroutine divides the two 16-bit numbers 
;* "dd8uH:dd8uL" (dividend) and "dv16uh:dv16ul" (divisor). 
;* The result is placed in "dres16uh:dres16ul" and the remainder in
;* "drem16uh:drem16ul".
;*  
;* Number of words      :19
;* Number of cycles     :235/251 (Min/Max)
;* Low registers used   :2 (drem16ul,drem16uh)
;* High registers used  :5 (dres16ul/dd16ul,dres16uh/dd16uh,dv16ul,dv16uh,
;*                          dcnt16u)
;*
;***************************************************************************

;***** Subroutine Register Variables

.equ    drem16ul,r14
.equ    drem16uh,r15
.equ    dres16ul,r16
.equ    dres16uh,r17
.equ    dd16ul  ,r16
.equ    dd16uh  ,r17
.equ    dv16ul  ,r18
.equ    dv16uh  ,r19
.equ    dcnt16u ,r20

;***** Code

.global div16u
div16u: clr     drem16ul        ;clear remainder Low byte
	sub     drem16uh,drem16uh;clear remainder High byte and carry
	ldi     dcnt16u,17      ;init loop counter
d16u_1: rol     dd16ul          ;shift left dividend
	rol     dd16uh
	dec     dcnt16u         ;decrement counter
	brne    d16u_2          ;if done
	ret                     ;    return
d16u_2: rol     drem16ul        ;shift dividend into remainder
	rol     drem16uh
	sub     drem16ul,dv16ul ;remainder = remainder - divisor
	sbc     drem16uh,dv16uh ;
	brcc    d16u_3          ;if result negative
	add     drem16ul,dv16ul ;    restore remainder
	adc     drem16uh,dv16uh
	clc                     ;    clear carry to be shifted into result
	rjmp    d16u_1          ;else
d16u_3: sec                     ;    set carry to be shifted into result
	rjmp    d16u_1
;+
; **-pmul-pcode multiply.
;
; inputs:
;   r10     -> result (32-bits)
;   r12     = multiplicand
;   r14     = multiplier
;
;-

    .global pmul
pmul:   pcode_routine   3
    rcall r10reloc
    movw mc16u,r12
    movw mp16u,r14
    rcall mpy16u
    movw z,r10
    st z+,m16u0
    st z+,m16u1
    st z+,m16u2
    st z+,m16u3
    ret
;+
; **-pdiv-pcode divide
;
; inputs:
;   r10     -> result, remainder
;   r12     = dividend
;   r14     = divisor
;-

    .global pdiv
pdiv:   pcode_routine 3
    rcall r10reloc
    movw dd16u,r12
    movw dv16u,r14
    rcall div16u
    movw z,r10
    st z+,dres16ul
    st z+,dres16uh
    st z+,drem16ul
    st z+,drem16uh
    ret
#endif
