;--------------------------------------------------------------------------
;+
; **-purl2int-pcode ASCII to integer in receive packet buffer
;
; purl2int pval,poffset
;
; inputs:
;   poffset = sram word with offset in receive packet data area
;
; outputs:
;   pval = sram word pointer to get returned integer value
;   poffset = updated sram word with offset in receive packet data area
;-
;--------------------------------------------------------------------------
#define T1 (BP+0)
#define T2 (T1+2)
#define T3 (T2+2)
#define TEMPS_END (T3+2)
#define N_TEMPS (TEMPS_END-T1)

#define P1    TEMPS_END
#define P2    P1+2
#define P3    P2+2

#define pval       P1
#define poffset    P2
#define PU2INT_TMP P3
#define PU2INT_TMP2 T1
#define PU2INT_OFF T2
#define PU2INT_VAL T3

.text
    .global purl2int
purl2int: pcode_routine   2
    rcall pip_setup_r10reloc
    pjump purl2int_body

.section seeprom
purl2int_body:
    penter N_TEMPS
    pmovw PU2INT_OFF,[poffset]      ; get starting offset
    pclrw PU2INT_VAL                ; zero returned value

purl2int_loop:
    purl2s PU2INT_TMP,[PU2INT_OFF],1  ; get next character from packet buffer
    pjumpne purl2int_done             ; exit if none left
    pandwi PU2INT_TMP,0xff            ; mask for LSB (used as word later)

 ;;pcrlf
 ;;pprintv "[PU2INT_OFF ",[PU2INT_OFF]
 ;;pprintv " PU2INT_VAL ",[PU2INT_VAL]
 ;;pprintv " PU2INT_TMP ",[PU2INT_TMP]
 ;;pprint "]"

    pcmpbi PU2INT_TMP,'0'           ; check character
    pjumplo purl2int_done             ; exit if < '0'
    pcmpbi PU2INT_TMP,'9'+1         ; check character
    pjumphis purl2int_done            ; exit if > '9'

    pincw PU2INT_OFF                ; bump buffer address

    psubwi PU2INT_TMP,'0'           ; subtract '0' from character
    pmovwi PU2INT_TMP2,[PU2INT_VAL] ; tmp2 = value
    pshnw PU2INT_TMP2,1             ; tmp2 = (value << 2)
    pshnw PU2INT_VAL,3              ; value = (value << 8)
    paddwi PU2INT_VAL,[PU2INT_TMP2] ; value = (value << 8) + (value << 2)
    paddwi PU2INT_VAL,[PU2INT_TMP]  ; value = value * 10 + (char - '0')

    pjump purl2int_loop               ; back for more

purl2int_done:
    pmovwi [pval],[PU2INT_VAL]      ; return value
    pmovwi [poffset],[PU2INT_OFF]   ; return updated offset
    pretn N_TEMPS

