;--------------------------------------------------------------------------
;+
; **-purlparm-find parameter string within receive packet buffer.
;
; purlparm poffset,"string"
; inputs:
;   poffset   = address of word in sram to get offset in receive buffer
;   "string"  = address of string to search for (e.g., "PARM=")
;
; outputs:
;   flags Z=1 : match
;         Z=0 : mismatch
;   poffset = offset in receive buffer just after matching string
;-
;--------------------------------------------------------------------------

#define T1 (BP+0)
#define TEMPS_END (T1+2)
#define N_TEMPS (TEMPS_END-T1)

#define P1    TEMPS_END
#define P2    P1+2
#define P3    P2+2

#define poffset2  P1
#define SINDX_S   P2
#define SINDX_TMP P3
#define SINDX_OFF T1

#ifdef DEBUG
#define D
#else
#define D ;
#endif

.text
    .global purlparm
purlparm: pcode_routine 2

    rcall pip_setup_r10r12reloc
    penter N_TEMPS

    D pserial_debug

    pmovwi SINDX_OFF,[url_parms]    ; get index to start of parameters
    purl2s SINDX_TMP,[SINDX_OFF],1  ; get first character from packet buffer
    pjumpne purlparm_mismatch       ; exit if none
    pjump purlparm_first_param

  ;;pserial_debug
  ;;pprintv "\r\nstart SINDX_OFF=",[SINDX_OFF]
  ;;pprintv " SINDX_S=",[SINDX_S]
  ;;pprint [SINDX_S]
  ;;pcrlf

purlparm_loop:
    pwdr                            ; reset hardware watchdog timer

    purl2s SINDX_TMP,[SINDX_OFF],1  ; next character from packet buffer
    pjumpne purlparm_mismatch_x     ; exit if none

    D pprintv " SINDX_OFF=",[SINDX_OFF]
    D pprintv " CH=",[SINDX_TMP]
    D pcrlf
    pcmpbi SINDX_TMP,' '            ; check for SPACE
    pjumpeq purlparm_mismatch       ; exit if found
    pjumplo purlparm_mismatch       ; exit if less

    pcmpbi SINDX_TMP,'&'            ; check for '&' (begin new param)
    pjumpne purlparm_noparm         ; check next character of not '&'

purlparm_param:
  ;;;;;pprintv " SINDX_OFF=",[SINDX_OFF]
  ;;;;;pprintv " SINDX_S=",[SINDX_S]
  ;;;;;pcrlf
    ;;;ppushwi [SINDX_OFF]
    pincw SINDX_OFF                  ; bump offset
purlparm_first_param:
    D pprintv " purlparm purl2scmp ",[SINDX_OFF]
    D pprint ",'"
    D pprint [SINDX_S]
    D pprint "'\r\n"
    purl2scmp SINDX_OFF,[SINDX_S]     ; strings
    D pprintv " SINDX_S=",[SINDX_S]
    D pprintv " SINDX_OFF=",[SINDX_OFF]
    D pcrlf
    pjumpeq purlparm_match         ; if match then exit
    ;;;ppopw SINDX_OFF

purlparm_noparm:
    pincw SINDX_OFF                 ; bump offset
    pjump purlparm_loop             ; back for more

purlparm_mismatch_x:
  ;;pprintv "SINDX_OFF=",[SINDX_OFF]
  ;;pprint " end of loop exit\r\n"
purlparm_mismatch:
    D pprint "mismatch\r\n"
    pcmpwi SINDX_OFF,1             ; set Z=0
    pjump purlparm_exit

purlparm_match:
  ;;pprintv "\r\nSINDX_OFF=",[SINDX_OFF]
    D pprint " match\r\n"
purlparm_exit:

    pmovwi [poffset2],[SINDX_OFF]
    D pprintv " after ppurlparm*poffset2=>",[SINDX_OFF]
    D pcrlf

    pretn N_TEMPS
