#include    <stdio.h>
#include    <string.h>
#include    <io.h>
#include    <fcntl.h>
#include    <stdlib.h>
#include    <windows.h>
#include    "getopt.h"

static FILE *OutFp ;
static BOOLEAN bBinary = FALSE ;
static HANDLE hOutput = INVALID_HANDLE_VALUE ;

static void
vPrintFile(
    FILE    *fp
){
    char    Buffer[4096] ;
    DWORD   nb ;
    size_t  n ;

    while ((n=fread(Buffer,1,sizeof Buffer,fp)) > 0)  {
        if (hOutput != INVALID_HANDLE_VALUE)  {
            WriteFile(hOutput,Buffer,n,&nb,NULL) ;
            if (n != (int)nb)  {
                fprintf(stderr,"WriteFile error %lu\n",GetLastError()) ;
                exit(3) ;
            }
        }
        else  {
            if (fwrite(Buffer,1,n,OutFp) != n)  {
                perror("stdout") ;
                exit(3) ;
            }
        }
    }
}

static void
vWild(
    char    *sName
){
    WIN32_FIND_DATA FindData ;
    HANDLE  hFind ;
    FILE    *fp ;
    char    sFilePath[1024] ;
    char    *s ;
    int     iIndex = 0 ;

    s = strrchr(sName,'/') ;
    if (s == NULL)  {
        s = strrchr(sName,'\\') ;
        if (s == NULL)  {
            s = strrchr(sName,':') ;
        }
    }
    if (s != NULL)  {       /* had a path */
        iIndex = (s-sName)+1 ;
        strncpy(sFilePath,sName,iIndex) ;
    }
    hFind = FindFirstFile(sName,&FindData) ;
    if (hFind == INVALID_HANDLE_VALUE)  {
        fprintf(stderr,"Can't open %s\n",sName) ;
        exit(1);
    }
    else  {
        do  {
            strcpy(&sFilePath[iIndex],FindData.cFileName) ;
            fp = fopen(sFilePath,"r") ;
            if (fp == NULL)  {
                perror(sFilePath) ;
            }
            else  {
                vPrintFile(fp) ;
                fclose(fp) ;
            }
        } while (FindNextFile(hFind,&FindData)) ;
        FindClose(hFind) ;
    }
}

int
main(
    int argc,
    char *argv[]
){
    int     i ;
    int     c ;

#define CMD_SWITCHES "bo:c:?"
    OutFp = stdout ;
    while ((c = getopt(argc,argv,CMD_SWITCHES)) != EOF)  {
        switch (c)  {
            case 'b':  {
                bBinary = TRUE ;
                break ;
            }
            case 'o':  {
                OutFp = fopen(optarg,bBinary ? "wb" : "w") ;
                if (OutFp == NULL)  {
                    perror(optarg) ;
                    return(3) ;
                }
                break ;
            }
            case 'c':  {
                hOutput = CreateFile(
                    optarg,
                    GENERIC_WRITE,
                    FILE_SHARE_READ|FILE_SHARE_WRITE,
                    NULL,
                    OPEN_ALWAYS,
                    FILE_ATTRIBUTE_NORMAL,
                    NULL) ;
                if (hOutput == INVALID_HANDLE_VALUE)  {
                    fprintf(stderr,"CreateFile error %lu\n",GetLastError()) ;
                    return(3) ;
                }
                break ;
            }
            default:
            case '?':  {
                fprintf(stderr,
                    "Usage:\n\tcat [-" CMD_SWITCHES "] [file...]\n") ;
                return(10) ;
            }
        }
    }
    if (optind >= argc)  {
        if (bBinary)  {
            _setmode(fileno(stdin),O_BINARY) ;
        }
        vPrintFile(stdin) ;
    }
    else  {
        for (i=optind ; i<argc ; i++)  {
            vWild(argv[i]) ;
        }
    }
    return(0) ;
}
