//;-------------------------------------------------------------------------
//; Copyright (c) 2001 Steven Freyder, David Helland, and Bruce D. Lightner.
//; Use subject to licensing restrictions described in "PICOWEB SERVER
//; SOFTWARE LICENSE AGREEMENT DATED JULY 14, 1999" (See file LICENSE.tt).
//;-------------------------------------------------------------------------

#include <windows.h>
#include <windowsx.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include <stdio.h>

BOOL InstallDriver(IN SC_HANDLE SchSCManager, IN LPCTSTR DriverName, IN LPCTSTR ServiceExe)
{
    SC_HANDLE schService ;
    
    schService = CreateService(SchSCManager,          	/* SCManager database */
			       DriverName,           	/* service name */
			       DriverName,           	/* display name */
			       SERVICE_ALL_ACCESS,    	/* desired access */
			       SERVICE_KERNEL_DRIVER, 	/* service type */
			       SERVICE_DEMAND_START,  	/* start type */
			       SERVICE_ERROR_NORMAL,  	/* error control type */
			       ServiceExe,            	/* service's binary */
			       NULL,                  	/* no load ordering group */
			       NULL,                  	/* no tag identifier */
			       NULL,                  	/* no dependencies */
			       NULL,                  	/* LocalSystem account */
			       NULL                   	/* no password */
			      ) ;
    if (schService == NULL)  {
        return(FALSE) ;
    }

    CloseServiceHandle(schService) ;
    return(TRUE) ;
}

BOOL StartDriver(IN SC_HANDLE SchSCManager, IN LPCTSTR DriverName)
{
    SC_HANDLE schService ;
    BOOL ret ;

    schService = OpenService(SchSCManager,
			     DriverName,
			     SERVICE_ALL_ACCESS
			    ) ;
    if (schService == NULL)  {
        return(FALSE) ;
    }

    ret = StartService(schService, 0, NULL) 
	|| GetLastError() == ERROR_SERVICE_ALREADY_RUNNING ;

    CloseServiceHandle(schService) ;

    return(ret) ;
}


BOOL OpenDevice(IN LPCTSTR DriverName, HANDLE *lphDevice)
{
    TCHAR completeDeviceName[64] ;
    HANDLE hDevice ;

    /*
     * Create a \\.\XXX device name that CreateFile can use
     *
     * NOTE: We're making an assumption here that the driver
     *       has created a symbolic link using it's own name
     *       (i.e. if the driver has the name "XXX" we assume
     *       that it used IoCreateSymbolicLink to create a
     *       symbolic link "\DosDevices\XXX". Usually, there
     *       is this understanding between related apps/drivers.
     *
     *       An application might also peruse the DEVICEMAP
     *       section of the registry, or use the QueryDosDevice
     *       API to enumerate the existing symbolic links in the
     *       system.
     */
    _stprintf(completeDeviceName, TEXT("\\\\.\\%s"), DriverName) ;

    hDevice = CreateFile(completeDeviceName,
			 GENERIC_READ | GENERIC_WRITE,
			 0,
			 NULL,
			 OPEN_EXISTING,
			 FILE_ATTRIBUTE_NORMAL,
			 NULL
			) ;
    if (hDevice == ((HANDLE)-1))  {
        return(FALSE) ;
    }

    /* If user wants handle, give it to them.  Otherwise, just close it. */
    if (lphDevice)  {
	*lphDevice = hDevice ;
    }
    else  {
	CloseHandle(hDevice) ;
    }
    return(TRUE) ;
}


BOOL StopDriver(IN SC_HANDLE SchSCManager, IN LPCTSTR DriverName)
{
    SC_HANDLE schService ;
    BOOL ret ;
    SERVICE_STATUS serviceStatus ;

    schService = OpenService(SchSCManager, DriverName, SERVICE_ALL_ACCESS) ;
    if (schService == NULL)  {
        return(FALSE) ;
    }

    ret = ControlService(schService, SERVICE_CONTROL_STOP, &serviceStatus) ;

    CloseServiceHandle(schService) ;

    return(ret);
}


BOOL RemoveDriver(IN SC_HANDLE SchSCManager, IN LPCTSTR DriverName)
{
    SC_HANDLE schService ;
    BOOL ret ;

    schService = OpenService(SchSCManager,
			     DriverName,
			     SERVICE_ALL_ACCESS
			    ) ;

    if (schService == NULL)  {
        return(FALSE) ;
    }

    ret = DeleteService(schService) ;

    CloseServiceHandle(schService) ;

    return(ret) ;
}


BOOL UnloadDeviceDriver(const TCHAR *Name)
{
    SC_HANDLE schSCManager ;

    schSCManager = OpenSCManager(NULL,                	/* machine (NULL == local) */
				 NULL,                 	/* database (NULL == default) */
				 SC_MANAGER_ALL_ACCESS 	/* access required */
				) ;

    StopDriver(schSCManager, Name) ;
    RemoveDriver(schSCManager, Name) ;
    CloseServiceHandle(schSCManager) ;
    return(TRUE) ;
}


BOOL LoadDeviceDriver(const TCHAR *Name, const TCHAR *Path, HANDLE *lphDevice)
{
    SC_HANDLE schSCManager ;
    BOOL okay ;

    schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS) ;

    /* Remove previous instance */
    RemoveDriver(schSCManager, Name) ;

    /* Ignore success of installation: it may already be installed. */
    InstallDriver(schSCManager, Name, Path) ;

    /* Ignore success of start: it may already be started. */
    StartDriver(schSCManager, Name) ;

    /* Do make sure we can open it. */
    okay = OpenDevice(Name, lphDevice) ;

    CloseServiceHandle(schSCManager) ;

    return(okay) ;
}
